/*
 *  Copyright (c) 2019 by flomesh.io
 *
 *  Unless prior written consent has been obtained from the copyright
 *  owner, the following shall not be allowed.
 *
 *  1. The distribution of any source codes, header files, make files,
 *     or libraries of the software.
 *
 *  2. Disclosure of any source codes pertaining to the software to any
 *     additional parties.
 *
 *  3. Alteration or removal of any notices in or on the software or
 *     within the documentation included within the software.
 *
 *  ALL SOURCE CODE AS WELL AS ALL DOCUMENTATION INCLUDED WITH THIS
 *  SOFTWARE IS PROVIDED IN AN “AS IS” CONDITION, WITHOUT WARRANTY OF ANY
 *  KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 *  OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 *  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 *  CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 *  TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 *  SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include "prometheus.hpp"
#include "metrics.hpp"

#include <sstream>

NS_BEGIN

//
// Prometheus
//

Prometheus::Prometheus() {
}

Prometheus::~Prometheus() {
}

auto Prometheus::help() -> std::list<std::string> {
  return {
    "Dumps counter data in Prometheus format",
  };
}

auto Prometheus::clone() -> Module* {
  return new Prometheus();
}

void Prometheus::pipe(
  std::shared_ptr<Context> ctx,
  std::unique_ptr<Object> obj,
  Object::Receiver out
) {
  if (obj->is<SessionStart>() || obj->is<SessionEnd>()) {
    out(std::move(obj));

  } else if (obj->is<MessageEnd>()) {
    std::stringstream buf;
    Metrics::dump(buf);
    out(make_object<MessageStart>());
    out(make_object<Data>(buf.str()));
    out(make_object<MessageEnd>());
  }
}

NS_END
